/**
 * @projectDescription publiquiz_basics.js
 * Plugin jQuery for quiz choices.
 *
 * @author prismallia.fr
 * @version 0.1
 * $Id: publiquiz_pip.js 488554215a1d 2015/04/21 14:08:28 Patrick $
 */

/**
 *
 * Copyright (C) Prismallia, Paris, 2015. All rights reserved.
 *
 * This program is free software. You can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */


/*jshint globalstrict: true*/
/*global jQuery: true */
/*global setTimeout: true */


/******************************************************************************
 *
 *                                  Pip
 *
******************************************************************************/

(function ($) {

"use strict";

$.publiquiz.question.pip = {

    /**
     * Configure quiz.
     */
    pipConfigure: function($quiz) {
    },

    /**
     * Set event click.
     *
     * @param {Object} jquery Object quiz.
     */
    pipEnable: function($quiz) {
        var prefix = $quiz.data("prefix");

        // Event "draggable" on item
        $quiz.find("."+prefix+"Item").prop("draggable", true);
        $.publiquiz.question.setDraggableItem($quiz, $quiz.find("."+prefix+"Item"), "");
    },

    /**
     * Remove events listener on quiz.
     *
     * @params {Object} $quiz : object jquery quiz.
     */
    pipDisable: function($quiz) {
        var prefix = $quiz.data("prefix");

        var evtStart = "mousedown touchstart";
        var $items = $quiz.find("."+prefix+"Item");
        $items.unbind(evtStart);
        $items.prop("draggable", false);
    },

    /**
     * Display help of quiz.
     *
     * @params {Object} $quiz : object jquery quiz.
     */
    pipHelp: function($quiz) {
        $.publiquiz.question.displayHelp($quiz);
    },

    /**
     * Retry quiz.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     */
    pipRetry: function($quiz) {
        $.publiquiz.question.retryQuizCmp($quiz);
    },

    /**
     * Compute score of quiz.
     *
     * @params {Object} $quiz : object jquery quiz.
     * @return {Dictionnary}.
     */
    pipComputeScore: function($quiz) {
        var result = {};
        var noMark = false;
        if ($quiz.data("engine-options") &&
                $quiz.data("engine-options").search("nomark") > -1 )
            noMark = true;

        if (noMark) {
            result.score = 0;
            result.total = 0;
            return result;
        }

        var quzId = $quiz.data("quiz-id");
        var prefix = $quiz.data("prefix");

        var total = 0;
        var score = 0;
        var res = $.publiquiz.question.correction($quiz.find("#"+quzId+"_correct"));
        $.each(res, function(key, value) {
            total++;
            var $dropbox = $quiz.find("#"+quzId+"_"+key);
            var $item = $dropbox.find("."+prefix+"Item");
            if ($item.length > 0) {
                var data = $item.data("item-value");
                if (data == value)
                    score += 1;
            } else if ($item.length === 0 && value === "") {
                score += 1;
            }
        });

        result.score = score;
        result.total = total;
        return result;
    },

    /**
     * Display score of quiz.
     *
     * @params {Object} $quiz : object jquery quiz.
     */
    pipScore: function($quiz) {
    },

    /**
     * Display quiz text answer.
     *
     * @params {Object} $quiz : object jquery quiz.
     */
    pipTextAnswer: function($quiz) {
        $.publiquiz.question.displayTextAnswer($quiz);
    },

    /**
     * Insert user answers in html
     *
     * @params {Object} $quiz : object jquery quiz.
     */
    pipInsertUserAnswers: function($quiz) {
        var answer = "";
        var quzId = $quiz.data("quiz-id");
        var prefix = $quiz.data("prefix");
        $quiz.find("."+prefix+"Drop").each( function() {
            var $dropbox = $(this);
            var key = $dropbox.attr("id");
            key = key.substring(key.length - 3, key.length);
            $dropbox.find("."+prefix+"Item").each( function() {
                var $item = $(this);
                var value = $item.data("item-value");
                if (answer !== "")
                    answer += "::";
                answer += key + value;
            });
        });

        $.publiquiz.question.writeUserAnswers($quiz, quzId, answer);
    },

    /**
     * Display right/user answer for quiz.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     * @params {String} mode : display correct/user answer.
     */
    pipQuizAnswer: function($quiz, mode) {
        var quzId = $quiz.data("quiz-id");
        var prefix = $quiz.data("prefix");
        /*var correctionOnly = $quiz.data("display-correction-only");*/
        var isMultiple = false;
        if ($quiz.data("engine-options") &&
                $quiz.data("engine-options").search("multiple") > -1 )
            isMultiple = true;
        var answers = $.publiquiz.question.correction($quiz.find("#"+quzId+mode));
        var $items = $quiz.find("#"+quzId+"_items");
        var userAnswers = null;
        if(mode == "_correct")
            userAnswers = $.publiquiz.question.correctionCategories($quiz.find("#"+quzId+"_user"));

        // On enleve la correction
        if (isMultiple) {
            $quiz.find("."+prefix+"ItemDropped").remove();
        } else {
            $quiz.find("."+prefix+"ItemDropped").each( function() {
                var $item = $(this);
                $item.appendTo($items)
                    .removeClass(prefix+"ItemDropped " +
                        prefix+"ItemImageDropped");
            });
        }

        // On place la correction en deplacant les items
        // ou bien en les clonant si "isMultiple" est a true
        $.each(answers, function(key, value) {
            var $dropbox = $quiz.find("#"+quzId+"_"+key);
            $dropbox.text("");
            $dropbox.removeClass("answerOk answerKo");
            var $item = $items.find("."+prefix+"Item").filter("[data-item-value=\""+value+"\"]");
            if (isMultiple)
                $item = $item.clone();
            //if (mode == "_correct") {
            //    if (correctionOnly) {
            //        $dropbox.addClass("answerOk");
            //    } else {
            //        if ((!userAnswers[key] && value !== "") ||
            //            (userAnswers[key] && value === "") ||
            //            (userAnswers[key] && userAnswers[key] != value))
            //            $dropbox.addClass("answerKo");
            //        else
            //            $dropbox.addClass("answerOk");
            //    }
            //}
            $item.appendTo($dropbox)
                .addClass(prefix+"ItemDropped " +
                    prefix+"ItemImageDropped");
        });
    },

    /**
     * Verify user answer.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     */
    pipVerifyUserAnswer: function($quiz) {
        $.publiquiz.question.pip._verifyAnswer($quiz, "user");
    },

    /**
     * Verify user answer.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     */
    pipVerifyRightAnswer: function($quiz) {
        $.publiquiz.question.pip._verifyAnswer($quiz, "right");
    },

    /**
     * Verify user answer.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     */
    pipVerifyFullAnswer: function($quiz) {
        $.publiquiz.question.pip._verifyAnswer($quiz, "full");
    },


    /**********************************************************************
     *                          Private Library
    **********************************************************************/

    /**
     * Verify answer.
     *
     * @params {Object} $quiz : object jquery publiquiz.
     * @params {String} mode.
     */
    _verifyAnswer: function($quiz, mode) {
        var quzId = $quiz.data("quiz-id");
        var prefix = $quiz.data("prefix");

        var rightAnswers = $.publiquiz.question.correction($quiz.find("#"+quzId+"_correct"));
        var userAnswers = $.publiquiz.question.correction($quiz.find("#"+quzId+"_user"));
        $.each(rightAnswers, function(key, value) {
            var $dropbox = $quiz.find("#"+quzId+"_"+key);
            var $item = $dropbox.find("."+prefix+"Item");
            if ($item.length > 0) {
                var data = $item.data("item-value");
                if (mode == "user") {
                    if (data == value)
                        $dropbox.addClass("answerOk");
                    else
                        $dropbox.addClass("answerKo");
                } else if (mode == "right") {
                    if (data == value)
                        $dropbox.addClass("answerOk");
                } else {
                    if ((!userAnswers[key] && value !== "") ||
                        (userAnswers[key] && value === "") ||
                        (userAnswers[key] && userAnswers[key] != value))
                        $dropbox.addClass("answerKo");
                    else
                        $dropbox.addClass("answerOk");
                }


            }
        });

        var duration = $.publiquiz.defaults.verifyDuration;
        if (duration < 0) {
            var timer = setTimeout($.publiquiz.question.clearVerify, duration);
            $.publiquiz.question.timers.push(timer);
        }
    }
};

// Register function
$.publiquiz.question.register("pip", {
        configure: $.publiquiz.question.pip.pipConfigure,
        enable: $.publiquiz.question.pip.pipEnable,
        disable: $.publiquiz.question.pip.pipDisable,
        help: $.publiquiz.question.pip.pipHelp,
        retry: $.publiquiz.question.pip.pipRetry,
        textAnswer: $.publiquiz.question.pip.pipTextAnswer,
        insertUserAnswers: $.publiquiz.question.pip.pipInsertUserAnswers,
        quizAnswer: $.publiquiz.question.pip.pipQuizAnswer,
        verifyUserAnswer: $.publiquiz.question.pip.pipVerifyUserAnswer,
        verifyRightAnswer: $.publiquiz.question.pip.pipVerifyRightAnswer,
        verifyFullAnswer: $.publiquiz.question.pip.pipVerifyFullAnswer,
        computeScore: $.publiquiz.question.pip.pipComputeScore,
        quizScore: $.publiquiz.question.pip.pipScore
    });

}(jQuery));
